@@ -1,19 +1,19 @@
 from __future__ import annotations
 
-import os
 from collections.abc import Iterable
 from functools import partial
 from math import ceil
 from operator import getitem
 from threading import Lock
-from typing import TYPE_CHECKING, Iterable, Literal
+from typing import TYPE_CHECKING, Iterable, Literal, overload
 
 import numpy as np
 import pandas as pd
 
 import dask.array as da
-from dask.base import tokenize
+from dask.base import is_dask_collection, tokenize
 from dask.blockwise import BlockwiseDepDict, blockwise
+from dask.dataframe.backends import dataframe_creation_dispatch
 from dask.dataframe.core import (
     DataFrame,
     Index,
@@ -25,7 +25,6 @@ from dask.dataframe.core import (
     new_dd_object,
 )
 from dask.dataframe.io.utils import DataFrameIOFunction
-from dask.dataframe.shuffle import set_partition
 from dask.dataframe.utils import (
     check_meta,
     insert_meta_param_description,
@@ -35,7 +34,7 @@ from dask.dataframe.utils import (
 from dask.delayed import Delayed, delayed
 from dask.highlevelgraph import HighLevelGraph
 from dask.layers import DataFrameIOLayer
-from dask.utils import M, _deprecated, funcname, is_arraylike
+from dask.utils import M, funcname, is_arraylike
 
 if TYPE_CHECKING:
     import distributed
@@ -153,6 +152,30 @@ def from_array(x, chunksize=50000, columns=None, meta=None):
     return new_dd_object(dsk, name, meta, divisions)
 
 
+@overload
+def from_pandas(
+    data: pd.DataFrame,
+    npartitions: int | None = None,
+    chunksize: int | None = None,
+    sort: bool = True,
+    name: str | None = None,
+) -> DataFrame:
+    ...
+
+
+# We ignore this overload for now until pandas-stubs can be added.
+# See https://github.com/dask/dask/issues/9220
+@overload
+def from_pandas(  # type: ignore
+    data: pd.Series,
+    npartitions: int | None = None,
+    chunksize: int | None = None,
+    sort: bool = True,
+    name: str | None = None,
+) -> Series:
+    ...
+
+
 def from_pandas(
     data: pd.DataFrame | pd.Series,
     npartitions: int | None = None,
@@ -180,11 +203,13 @@ def from_pandas(
     data : pandas.DataFrame or pandas.Series
         The DataFrame/Series with which to construct a Dask DataFrame/Series
     npartitions : int, optional
-        The number of partitions of the index to create. Note that depending on
-        the size and index of the dataframe, the output may have fewer
-        partitions than requested.
+        The number of partitions of the index to create. Note that if there
+        are duplicate values or insufficient elements in ``data.index``, the
+        output may have fewer partitions than requested.
     chunksize : int, optional
-        The number of rows per index partition to use.
+        The desired number of rows per index partition to use. Note that
+        depending on the size and index of the dataframe, actual partition
+        sizes may vary.
     sort: bool
         Sort the input by index first to obtain cleanly divided partitions
         (with known divisions).  If False, the input will not be sorted, and
@@ -232,23 +257,22 @@ def from_pandas(
     if not has_parallel_type(data):
         raise TypeError("Input must be a pandas DataFrame or Series.")
 
-    if (npartitions is None) == (none_chunksize := (chunksize is None)):
+    if (npartitions is None) == (chunksize is None):
         raise ValueError("Exactly one of npartitions and chunksize must be specified.")
 
     nrows = len(data)
 
-    if none_chunksize:
+    if chunksize is None:
         if not isinstance(npartitions, int):
             raise TypeError(
                 "Please provide npartitions as an int, or possibly as None if you specify chunksize."
             )
-        chunksize = int(ceil(nrows / npartitions))
     elif not isinstance(chunksize, int):
         raise TypeError(
             "Please provide chunksize as an int, or possibly as None if you specify npartitions."
         )
 
-    name = name or ("from_pandas-" + tokenize(data, chunksize))
+    name = name or ("from_pandas-" + tokenize(data, chunksize, npartitions))
 
     if not nrows:
         return new_dd_object({(name, 0): data}, name, data, [None, None])
@@ -263,9 +287,14 @@ def from_pandas(
         if not data.index.is_monotonic_increasing:
             data = data.sort_index(ascending=True)
         divisions, locations = sorted_division_locations(
-            data.index, chunksize=chunksize
+            data.index,
+            npartitions=npartitions,
+            chunksize=chunksize,
         )
     else:
+        if chunksize is None:
+            assert isinstance(npartitions, int)
+            chunksize = int(ceil(nrows / npartitions))
         locations = list(range(0, nrows, chunksize)) + [len(data)]
         divisions = [None] * len(locations)
 
@@ -276,169 +305,48 @@ def from_pandas(
     return new_dd_object(dsk, name, data, divisions)
 
 
-@_deprecated(after_version="2022.02.1")
-def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock, **kwargs):
-    """Read BColz CTable into a Dask Dataframe
-
-    BColz is a fast on-disk compressed column store with careful attention
-    given to compression.  https://bcolz.readthedocs.io/en/latest/
+@dataframe_creation_dispatch.register_inplace("pandas")
+def from_dict(data, npartitions, orient="columns", dtype=None, columns=None):
+    """
+    Construct a Dask DataFrame from a Python Dictionary
 
     Parameters
     ----------
-    x : bcolz.ctable
-    chunksize : int, optional
-        The size(rows) of blocks to pull out from ctable.
-    categorize : bool, defaults to True
-        Automatically categorize all string dtypes
-    index : string, optional
-        Column to make the index
-    lock: bool or Lock
-        Lock to use when reading or False for no lock (not-thread-safe)
+    data : dict
+        Of the form {field : array-like} or {field : dict}.
+    npartitions : int
+        The number of partitions of the index to create. Note that depending on
+        the size and index of the dataframe, the output may have fewer
+        partitions than requested.
+    orient : {'columns', 'index', 'tight'}, default 'columns'
+        The "orientation" of the data. If the keys of the passed dict
+        should be the columns of the resulting DataFrame, pass 'columns'
+        (default). Otherwise if the keys should be rows, pass 'index'.
+        If 'tight', assume a dict with keys
+        ['index', 'columns', 'data', 'index_names', 'column_names'].
+    dtype: bool
+        Data type to force, otherwise infer.
+    columns: string, optional
+        Column labels to use when ``orient='index'``. Raises a ValueError
+        if used with ``orient='columns'`` or ``orient='tight'``.
 
-    See Also
+    Examples
     --------
-    from_array: more generic function not optimized for bcolz
+    >>> import dask.dataframe as dd
+    >>> ddf = dd.from_dict({"num1": [1, 2, 3, 4], "num2": [7, 8, 9, 10]}, npartitions=2)
     """
-    if lock is True:
-        lock = Lock()
 
-    import bcolz
-
-    import dask.array as da
-
-    if isinstance(x, str):
-        x = bcolz.ctable(rootdir=x)
-    bc_chunklen = max(x[name].chunklen for name in x.names)
-    if chunksize is None and bc_chunklen > 10000:
-        chunksize = bc_chunklen
-
-    categories = dict()
-    if categorize:
-        for name in x.names:
-            if (
-                np.issubdtype(x.dtype[name], np.string_)
-                or np.issubdtype(x.dtype[name], np.unicode_)
-                or np.issubdtype(x.dtype[name], np.object_)
-            ):
-                a = da.from_array(x[name], chunks=(chunksize * len(x.names),))
-                categories[name] = da.unique(a).compute()
-
-    columns = tuple(x.dtype.names)
-    divisions = tuple(range(0, len(x), chunksize))
-    divisions = divisions + (len(x) - 1,)
-    if x.rootdir:
-        token = tokenize(
-            (x.rootdir, os.path.getmtime(x.rootdir)),
-            chunksize,
-            categorize,
-            index,
-            kwargs,
+    collection_types = {type(v) for v in data.values() if is_dask_collection(v)}
+    if collection_types:
+        raise NotImplementedError(
+            "from_dict doesn't currently support Dask collections as inputs. "
+            f"Objects of type {collection_types} were given in the input dict."
         )
-    else:
-        token = tokenize(
-            (id(x), x.shape, x.dtype), chunksize, categorize, index, kwargs
-        )
-    new_name = "from_bcolz-" + token
 
-    dsk = {
-        (new_name, i): (
-            dataframe_from_ctable,
-            x,
-            (slice(i * chunksize, (i + 1) * chunksize),),
-            columns,
-            categories,
-            lock,
-        )
-        for i in range(0, int(ceil(len(x) / chunksize)))
-    }
-
-    meta = dataframe_from_ctable(x, slice(0, 0), columns, categories, lock)
-    result = DataFrame(dsk, new_name, meta, divisions)
-
-    if index:
-        assert index in x.names
-        a = da.from_array(x[index], chunks=(chunksize * len(x.names),))
-        q = np.linspace(0, 100, len(x) // chunksize + 2)
-        divisions = tuple(da.percentile(a, q).compute())
-        return set_partition(result, index, divisions, **kwargs)
-    else:
-        return result
-
-
-def dataframe_from_ctable(x, slc, columns=None, categories=None, lock=lock):
-    """Get DataFrame from bcolz.ctable
-
-    Parameters
-    ----------
-    x: bcolz.ctable
-    slc: slice
-    columns: list of column names or None
-
-    >>> import bcolz
-    >>> x = bcolz.ctable([[1, 2, 3, 4], [10, 20, 30, 40]], names=['a', 'b'])
-    >>> dataframe_from_ctable(x, slice(1, 3))
-       a   b
-    1  2  20
-    2  3  30
-
-    >>> dataframe_from_ctable(x, slice(1, 3), columns=['b'])
-        b
-    1  20
-    2  30
-
-    >>> dataframe_from_ctable(x, slice(1, 3), columns='b')
-    1    20
-    2    30
-    Name: b, dtype: int...
-
-    """
-    import bcolz
-
-    if columns is None:
-        columns = x.dtype.names
-    if isinstance(columns, tuple):
-        columns = list(columns)
-
-    x = x[columns]
-    if type(slc) is slice:
-        start = slc.start
-        stop = slc.stop if slc.stop < len(x) else len(x)
-    else:
-        start = slc[0].start
-        stop = slc[0].stop if slc[0].stop < len(x) else len(x)
-    idx = pd.Index(range(start, stop))
-
-    if lock:
-        lock.acquire()
-    try:
-        if isinstance(x, bcolz.ctable):
-            chunks = [x[name][slc] for name in columns]
-            if categories is not None:
-                chunks = [
-                    pd.Categorical.from_codes(
-                        np.searchsorted(categories[name], chunk), categories[name], True
-                    )
-                    if name in categories
-                    else chunk
-                    for name, chunk in zip(columns, chunks)
-                ]
-            result = pd.DataFrame(
-                dict(zip(columns, chunks)), columns=columns, index=idx
-            )
-
-        elif isinstance(x, bcolz.carray):
-            chunk = x[slc]
-            if categories is not None and columns and columns in categories:
-                chunk = pd.Categorical.from_codes(
-                    np.searchsorted(categories[columns], chunk),
-                    categories[columns],
-                    True,
-                )
-            result = pd.Series(chunk, name=columns, index=idx)
-    finally:
-        if lock:
-            lock.release()
-    return result
+    return from_pandas(
+        pd.DataFrame.from_dict(data, orient, dtype, columns),
+        npartitions,
+    )
 
 
 def _partition_from_array(data, index=None, initializer=None, **kwargs):
@@ -537,14 +445,20 @@ def from_dask_array(x, columns=None, index=None, meta=None):
         # Create a mapping of chunk number in the incoming array to
         # (start row, stop row) tuples. These tuples will be used to create a sequential
         # RangeIndex later on that is continuous over the whole DataFrame.
+        n_elements = sum(x.chunks[0])
         divisions = [0]
         stop = 0
         index_mapping = {}
         for i, increment in enumerate(x.chunks[0]):
             stop += increment
             index_mapping[(i,)] = (divisions[i], stop)
+
+            # last division corrected, even if there are empty chunk(s) at the end
+            if stop == n_elements:
+                stop -= 1
+
             divisions.append(stop)
-        divisions[-1] -= 1
+
         arrays_and_indices.extend([BlockwiseDepDict(mapping=index_mapping), "i"])
 
     if is_series_like(meta):
@@ -749,10 +663,10 @@ def sorted_division_locations(seq, npartitions=None, chunksize=None):
 
     >>> L = ['A', 'A', 'A', 'A', 'B', 'B', 'B', 'C']
     >>> sorted_division_locations(L, chunksize=3)
-    (['A', 'B', 'C'], [0, 4, 8])
+    (['A', 'B', 'C', 'C'], [0, 4, 7, 8])
 
     >>> sorted_division_locations(L, chunksize=2)
-    (['A', 'B', 'C'], [0, 4, 8])
+    (['A', 'B', 'C', 'C'], [0, 4, 7, 8])
 
     >>> sorted_division_locations(['A'], chunksize=2)
     (['A', 'A'], [0, 1])
@@ -760,26 +674,97 @@ def sorted_division_locations(seq, npartitions=None, chunksize=None):
     if (npartitions is None) == (chunksize is None):
         raise ValueError("Exactly one of npartitions and chunksize must be specified.")
 
+    # Find unique-offset array (if duplicates exist).
+    # Note that np.unique(seq) should work in all cases
+    # for newer versions of numpy/pandas
+    seq_unique = seq.unique() if hasattr(seq, "unique") else np.unique(seq)
+    duplicates = len(seq_unique) < len(seq)
+    enforce_exact = False
+    if duplicates:
+        offsets = (
+            # Avoid numpy conversion (necessary for dask-cudf)
+            seq.searchsorted(seq_unique, side="left")
+            if hasattr(seq, "searchsorted")
+            else np.array(seq).searchsorted(seq_unique, side="left")
+        )
+        enforce_exact = npartitions and len(offsets) >= npartitions
+    else:
+        offsets = seq_unique = None
+
+    # Define chunksize and residual so that
+    # npartitions can be exactly satisfied
+    # when duplicates is False
+    residual = 0
+    subtract_drift = False
     if npartitions:
-        chunksize = ceil(len(seq) / npartitions)
+        chunksize = len(seq) // npartitions
+        residual = len(seq) % npartitions
+        subtract_drift = True
 
-    positions = [0]
-    values = [seq[0]]
-    for pos in range(0, len(seq), chunksize):
-        if pos <= positions[-1]:
-            continue
-        while pos + 1 < len(seq) and seq[pos - 1] == seq[pos]:
-            pos += 1
-        values.append(seq[pos])
-        if pos == len(seq) - 1:
-            pos += 1
-        positions.append(pos)
+    def chunksizes(ind):
+        # Helper function to satisfy npartitions
+        return chunksize + int(ind < residual)
 
-    if positions[-1] != len(seq):
-        positions.append(len(seq))
-        values.append(seq[-1])
+    # Always start with 0th item in seqarr,
+    # and then try to take chunksize steps
+    # along the seqarr array
+    divisions = [seq[0]]
+    locations = [0]
+    i = chunksizes(0)
+    ind = None  # ind cache (sometimes avoids nonzero call)
+    drift = 0  # accumulated drift away from ideal chunksizes
+    divs_remain = npartitions - len(divisions) if enforce_exact else None
+    while i < len(seq):
+        # Map current position selection (i)
+        # to the corresponding division value (div)
+        div = seq[i]
+        # pos is the position of the first occurance of
+        # div (which is i when seq has no duplicates)
+        if duplicates:
+            # Note: cupy requires casts to `int` below
+            if ind is None:
+                ind = int((seq_unique == seq[i]).nonzero()[0][0])
+            if enforce_exact:
+                # Avoid "over-stepping" too many unique
+                # values when npartitions is approximately
+                # equal to len(offsets)
+                offs_remain = len(offsets) - ind
+                if divs_remain > offs_remain:
+                    ind -= divs_remain - offs_remain
+                    i = offsets[ind]
+                    div = seq[i]
+            pos = int(offsets[ind])
+        else:
+            pos = i
+        if div <= divisions[-1]:
+            # pos overlaps with divisions.
+            # Try the next element on the following pass
+            if duplicates:
+                ind += 1
+                # Note: cupy requires cast to `int`
+                i = int(offsets[ind]) if ind < len(offsets) else len(seq)
+            else:
+                i += 1
+        else:
+            # pos does not overlap with divisions.
+            # Append candidate pos/div combination, and
+            # take another chunksize step
+            if subtract_drift:
+                # Only subtract drift when user specified npartitions
+                drift = drift + ((pos - locations[-1]) - chunksizes(len(divisions) - 1))
+            if enforce_exact:
+                divs_remain -= 1
+            i = pos + max(1, chunksizes(len(divisions)) - drift)
+            divisions.append(div)
+            locations.append(pos)
+            ind = None
 
-    return values, positions
+    # The final element of divisions/locations
+    # will always be the same
+    divisions.append(seq[-1])
+    locations.append(len(seq))
+
+    return divisions, locations
 
 
 class _PackedArgCallable(DataFrameIOFunction):
@@ -892,7 +877,8 @@ def from_map(
         Whether to enforce at runtime that the structure of the DataFrame
         produced by ``func`` actually matches the structure of ``meta``.
         This will rename and reorder columns for each partition,
-        and will raise an error if this doesn't work or types don't match.
+        and will raise an error if this doesn't work,
+        but it won't raise if dtypes don't match.
     **kwargs:
         Key-word arguments to broadcast to each output partition. These
         same arguments will be passed to ``func`` for every output partition.
