@@ -17,7 +17,12 @@ import pytest
 import dask
 import dask.dataframe as dd
 from dask.base import compute_as_if_collection
-from dask.dataframe._compat import PANDAS_GT_120, assert_categorical_equal, tm
+from dask.dataframe._compat import (
+    PANDAS_GT_120,
+    PANDAS_GT_140,
+    assert_categorical_equal,
+    tm,
+)
 from dask.dataframe.shuffle import (
     _noop,
     maybe_buffered_partd,
@@ -179,21 +184,53 @@ def test_partitioning_index_categorical_on_values():
     "npartitions", [1, 4, 7, pytest.param(23, marks=pytest.mark.slow)]
 )
 def test_set_index_general(npartitions, shuffle_method):
+    names = ["alice", "bob", "ricky"]
     df = pd.DataFrame(
-        {"x": np.random.random(100), "y": np.random.random(100) // 0.2},
+        {
+            "x": np.random.random(100),
+            "y": np.random.random(100) // 0.2,
+            "z": np.random.choice(names, 100),
+        },
         index=np.random.random(100),
     )
+    # Ensure extension dtypes work
+    # NOTE: Older version of pandas have known issues with extension dtypes.
+    # We generally expect extension dtypes to work well when using `pandas>=1.4.0`.
+    if PANDAS_GT_140:
+        df = df.astype({"x": "Float64", "z": "string"})
 
     ddf = dd.from_pandas(df, npartitions=npartitions)
 
     assert_eq(df.set_index("x"), ddf.set_index("x", shuffle=shuffle_method))
     assert_eq(df.set_index("y"), ddf.set_index("y", shuffle=shuffle_method))
+    assert_eq(df.set_index("z"), ddf.set_index("z", shuffle=shuffle_method))
     assert_eq(df.set_index(df.x), ddf.set_index(ddf.x, shuffle=shuffle_method))
     assert_eq(
         df.set_index(df.x + df.y), ddf.set_index(ddf.x + ddf.y, shuffle=shuffle_method)
     )
     assert_eq(df.set_index(df.x + 1), ddf.set_index(ddf.x + 1, shuffle=shuffle_method))
-    assert_eq(df.set_index(df.index), ddf.set_index(ddf.index, shuffle=shuffle_method))
+
+
+@pytest.mark.skipif(
+    not PANDAS_GT_140, reason="Only test `string[pyarrow]` on recent versions of pandas"
+)
+@pytest.mark.parametrize(
+    "string_dtype", ["string[python]", "string[pyarrow]", "object"]
+)
+def test_set_index_string(shuffle_method, string_dtype):
+    if string_dtype == "string[pyarrow]":
+        pytest.importorskip("pyarrow")
+    names = ["alice", "bob", "ricky"]
+    df = pd.DataFrame(
+        {
+            "x": np.random.random(100),
+            "y": np.random.choice(names, 100),
+        },
+        index=np.random.random(100),
+    )
+    df = df.astype({"y": string_dtype})
+    ddf = dd.from_pandas(df, npartitions=10)
+    assert_eq(df.set_index("y"), ddf.set_index("y", shuffle=shuffle_method))
 
 
 def test_set_index_self_index(shuffle_method):
@@ -203,7 +240,8 @@ def test_set_index_self_index(shuffle_method):
     )
 
     a = dd.from_pandas(df, npartitions=4)
-    b = a.set_index(a.index, shuffle=shuffle_method)
+    with pytest.warns(UserWarning, match="this is a no-op"):
+        b = a.set_index(a.index, shuffle=shuffle_method)
     assert a is b
 
     assert_eq(b, df.set_index(df.index))
@@ -714,7 +752,7 @@ def test_set_index_interpolate_large_uint(engine):
 
     if engine == "cudf":
         gdf = cudf.from_pandas(df)
-        d = dask_cudf.from_cudf(gdf, npartitions=2)
+        d = dask_cudf.from_cudf(gdf, npartitions=1)
     else:
         d = dd.from_pandas(df, 1)
 
@@ -727,7 +765,7 @@ def test_set_index_timezone():
     s_naive = pd.Series(pd.date_range("20130101", periods=3))
     s_aware = pd.Series(pd.date_range("20130101", periods=3, tz="US/Eastern"))
     df = pd.DataFrame({"tz": s_aware, "notz": s_naive})
-    d = dd.from_pandas(df, 2)
+    d = dd.from_pandas(df, npartitions=1)  # TODO: Use npartitions=2
 
     d1 = d.set_index("notz", npartitions=1)
     s1 = pd.DatetimeIndex(s_naive.values, dtype=s_naive.dtype)
@@ -749,7 +787,7 @@ def test_set_index_timezone():
         assert not d2.divisions[0] == s2badtype[0]
     else:
         with pytest.raises(TypeError):
-            d2.divisions[0] == s2badtype[0]
+            assert d2.divisions[0] == s2badtype[0]
 
 
 def test_set_index_npartitions():
@@ -878,7 +916,6 @@ def test_set_index_sorted_min_max_same():
 
 def test_set_index_empty_partition():
     test_vals = [1, 2, 3]
-
     converters = [int, float, str, lambda x: pd.to_datetime(x, unit="ns")]
 
     for conv in converters:
@@ -906,11 +943,17 @@ def test_set_index_on_empty():
 
         assert ddf.npartitions > 1
 
-        ddf = ddf[ddf.y > df.y.max()].set_index("x")
-        expected_df = df[df.y > df.y.max()].set_index("x")
+        actual = ddf[ddf.y > df.y.max()].set_index("x")
+        expected = df[df.y > df.y.max()].set_index("x")
 
-        assert assert_eq(ddf, expected_df, **CHECK_FREQ)
-        assert ddf.npartitions == 1
+        assert assert_eq(actual, expected, **CHECK_FREQ)
+        assert actual.npartitions == 1
+        assert all(pd.isnull(d) for d in actual.divisions)
+
+        actual = ddf[ddf.y > df.y.max()].set_index("x", sorted=True)
+        assert assert_eq(actual, expected, **CHECK_FREQ)
+        assert actual.npartitions == 1
+        assert all(pd.isnull(d) for d in actual.divisions)
 
 
 def test_set_index_categorical():
@@ -1177,6 +1220,17 @@ def test_set_index_overlap_2():
     assert ddf2.npartitions == 8
 
 
+def test_set_index_overlap_does_not_drop_rows_when_divisions_overlap():
+    # https://github.com/dask/dask/issues/9339
+    df = pd.DataFrame({"ts": [1, 1, 2, 2, 3, 3, 3, 3], "value": "abc"})
+    ddf = dd.from_pandas(df, npartitions=3)
+
+    expected = df.set_index("ts")
+    actual = ddf.set_index("ts", sorted=True)
+
+    assert_eq(expected, actual)
+
+
 def test_compute_current_divisions_nan_partition():
     # Compute divisions 1 null partition
     a = d[d.a > 3].sort_values("a")
